%%  Runs the model over a range of paramters. 
%% Alexey Ryabov 2014, 2015
%% please contact alexey.ryabov@uni-oldenburg.de if you need any help to use this code
%%
function [Params] = ...
    NSpecCompAdv_Cluster_Run_Model(SimulatID, ResDistr, Calculate)

% SimulatID type of the symulation, defines a set of paramters, which will be used for the symulation 
% (see different sections of the switch operator below)
% ResDistr resource distribution g - gradient, l - random locations, s - random supplies
% Set Calculate to 1 to run the model, to 0 to load saved data and plot results
%% example  NSpecCompAdv_Cluster_Run_Model('Diff001', 'g', 1)

close all

sp=30;  %number of species 

ReshuffleStep = 100;
SaveSteps = 100;


tmax = 200; %THE MAXIMAL modelling time

Params = [];

%default paramters for all symulations
ParamDefault.consump = 20;  

TimeSteps2Save = 10;


switch SimulatID
    case 'MetaComm0.1'
        Diff = 0.1;
        %         ResourceRangesLowHigh = ...
        %             [1, 40; 3, 38; 5.5, 35.5; 8, 33; 10.5, 30.5; 13, 28; 15.5, 25.5; 18, 23];
        %
        %         TraitRangesLowHigh = ...
        %             [1, 10; 2, 9; 3, 8; 5, 6];
        
        ResourceRangesLowHigh = ...
            [1, 40; 5.5, 35.5; 18, 23];
        
        TraitRangesLowHigh = ...
            [1, 10;  3, 8; 5, 6];
        
        
        for i = 1:size(ResourceRangesLowHigh, 1)
            for j = 1:size(TraitRangesLowHigh, 1)
                p = ParamDefault;
                p.ResLow = ResourceRangesLowHigh(i, 1);
                p.ResHigh = ResourceRangesLowHigh(i, 2);
                p.TraitLow = TraitRangesLowHigh(j, 1);
                p.TraitHigh = TraitRangesLowHigh(j, 2);
                p.ResDistr = ResDistr;
                p.Diff     = Diff;
                p.sp   = sp;                             %number of species
                p.specID = 1:sp;                         %use all species for the simulation
                                                         %you can define e.g. p.specID = [1, 5, 6] 
                p.tmax = tmax;
                suffix     = [num2str(p.ResLow) '_' num2str(p.ResHigh) '_' num2str(p.TraitLow) '_' num2str(p.TraitHigh)];
                p.DataFileName = ['Data/' SimulatID '/SP_coex' ResDistr suffix '_' num2str(tmax) '.dat'];
                if Diff > 0
                    [pathstr, name, ext] = fileparts(p.DataFileName);
                    p.DataFileName = [pathstr, '/' name, '_d', num2str(Diff)  ext];
                end
                if isempty(Params)
                    Params = p;
                else
                    Params(end + 1) = p;
                end
            end
        end
        
    case 'Diff001'
        
         
        ResourceRangesLowHigh = [1, 40];
        TraitRangesLowHigh    = [1, 10];
        
        Diff = [0, 0.1, 0.5, 1]; %%Set your dispersal for species
        DataFileName = ['Data/' SimulatID '/' ResDistr '_T' num2str(tmax) '.dat'];

        ParamName_field = 'Diff'; %% the name of the varying variable in the script
        ParamName_plot   = 'Dispersal'; %% how this variable should be called in figures
        
       
        for i = 1:length(Diff)
                %set paramter values for each simulation
                p = ParamDefault;
                p.ResLow = ResourceRangesLowHigh(1, 1);  %the minimal values of the resource supplies
                p.ResHigh = ResourceRangesLowHigh(1, 2); %the maximla values of the resource supplies
                p.TraitLow = TraitRangesLowHigh(1, 1);   %the minimal R* value
                p.TraitHigh = TraitRangesLowHigh(1, 2);  %the maximal R* value
                p.ResDistr = ResDistr;                   %the resource distribution
                p.Diff     = Diff(i);                       %the species dispersal rate
                p.tmax = tmax;                           %the simulation time
                p.sp   = sp;                             %number of species
                p.specID = 1:sp;                         %use all species for the simulation
                                          %you can define e.g. p.specID = [1, 5, 6] 
                                          %to select which species will be used
                if isempty(Params)
                    Params = p;
                else
                    Params(end + 1) = p;
                end
        end
        
        
        
end

%% Spatial dynamics 
if Calculate
    Results = [];
    for iParam = 1:length(Params) %% Loop over one parameter
        iParam
        %initialize the model
        [Int, dx, L, CellNumb, fl, dt, tmin, tmax, t, tsteps, TimeSteps, ...
            ReshuffleTime, res, D, S, R, ResDyn, d, r, m, R_eq, c, N,DispRes] = ...
            Model_Params (Params(iParam), ReshuffleStep);
        TSpan = linspace(tmin, tmax, TimeSteps2Save);

        %run the model
        [TSpan, N_Dyn, R_Dyn] = run_MetaCom...
            (N, c, R_eq, r, m, d, DispRes, R, S, D, TSpan, fl, dx, L,ReshuffleTime,ReshuffleStep);

        %the format of the results    
        %N_Dyn X_bins x Y_bins x  Species  xTimeSteps
        %N_Dyn(x, y, Sp_number, t)
        %N_Dyn(x, y, Sp_number, :) plot the dynymics of species Sp_number
        %at the point x, y
        %N_Dyn(:, :, Sp_number, end) the final biomass of species Sp_number

        %R_Dyn X_bins x Y_bins x  Resources  x TimeSteps


        %%Create a structure to save the results
        Results(iParam).TSpan = TSpan;
        Results(iParam).N_Dyn = N_Dyn;
        Results(iParam).R_Dyn = R_Dyn;
        Results(iParam).Params = Params(iParam);
    end
    [pathstr, name, ext] = fileparts(DataFileName);
    mkdir(pathstr);
    save(DataFileName, 'Results', 'Params');  %save results
else
    load(DataFileName, '-mat');    %load data
end


%%calculate biodiversity indexes
for  iParam = 1:length(Params)
    %calculate the final global biomass
    FinalBiomass = zeros(Results(iParam).Params.sp, 1);
    for s = 1:Results(iParam).Params.sp
        Sp_FinalDistrib = Results(iParam).N_Dyn(:, :, s, end);
        FinalBiomass(s)  = sum(Sp_FinalDistrib(:));
    end
    Results(iParam).FinalBiomass =  FinalBiomass;
    
    %another variant to calcualte the biomass
    %Results(iParam).FinalBiomass =  squeeze(sum(sum(Results(iParam).N_Dyn(:, :, :, end))))
    
    %calculate the dynamics of the global biomass
    GloabalBiomass_dyn = zeros(Results(iParam).Params.sp,  length(Results(iParam).TSpan));
    for t = 1:length(Results(iParam).TSpan)
        for s = 1:Results(iParam).Params.sp
            Sp_Distrib = Results(iParam).N_Dyn(:, :, s, t);
            GloabalBiomass_dyn(s, t)  = sum(Sp_Distrib(:));
        end
    end
    Results(iParam).GloabalBiomass_dyn =  GloabalBiomass_dyn;
end


%% plot results and calculate biodiversity etc indexes 
% To plot the results you work only with the variables which you saved!!
% (now only Results)

%plot the dynamics of species biomass in different simulations
Fig1 = figure(1);
set(Fig1, 'Position', [100, 100, 400, 800]);

for iParam = 1:length(Params)
    subplot(length(Params), 1, iParam);
    
    clrs = jet(Results(iParam).Params.sp);
    hold on
    for s = 1:Results(iParam).Params.sp
       plot(Results(iParam).TSpan, Results(iParam).GloabalBiomass_dyn(s, :), 'Color', clrs(s, :)) 
    end
    hold off
    title([ParamName_plot '=' num2str(Params(iParam).(ParamName_field))]);

end



